@@ -1,7 +1,7 @@
 use strict;
 use warnings;
 package CPAN::Uploader;
-our $VERSION = '0.001';
+our $VERSION = '0.002';
 
 # ABSTRACT: upload things to the CPAN
 
@@ -11,51 +11,36 @@ use HTTP::Request::Common qw(POST);
 use HTTP::Status;
 use LWP::UserAgent;
 
-
-sub log   { shift; print "$_\n" for @_ }
-sub debug { return unless $ENV{CPAN_UPLOAD_DEBUG}; shift->log(@_) }
-
 my $PAUSE_ADD_URI = 'http://pause.perl.org/pause/authenquery';
 
 
 sub upload_file {
   my ($self, $file, $arg) = @_;
 
+  Carp::confess("don't supply %arg when calling upload_file on an object")
+    if $arg and ref $self;
+
+  $self = $self->new($arg) if $arg;
+
   $self->log("registering upload with PAUSE web server");
 
-  # Create the agent we'll use to make the web requests
-  $self->debug("creating instance of LWP::UserAgent");
-
-  my $agent = LWP::UserAgent->new() || die "Failed to create UserAgent: $!";
+  my $agent = LWP::UserAgent->new;
   $agent->agent($self . q{/} . $self->VERSION);
 
-  $agent->proxy(['http'], $arg->{http_proxy}) if $arg->{http_proxy};
-
-  # Post an upload message to the PAUSE web site for each file
-  my $basename = File::Basename::basename($file);
-
-  open(my $fh, $file) or die "Failed to open $file: $!";
-  my $contents = do { local $/; <$fh> };
-  close($fh);
-
-  # Create the request to add the file
-  my $form = {
-    HIDDENNAME                        => $arg->{user},
-    CAN_MULTIPART                     => 1,
-    pause99_add_uri_upload            => $basename,
-    SUBMIT_pause99_add_uri_httpupload => " Upload this file from my disk ",
-    pause99_add_uri_uri               => "",
-    pause99_add_uri_httpupload        => [$file],
-  };
-
-  if ($arg->{directory}) {
-    $form->{pause99_add_uri_subdirtext} = $arg->{directory};
-  }
+  $agent->proxy(http => $arg->{http_proxy}) if $arg->{http_proxy};
 
   my $request = POST(
     $PAUSE_ADD_URI,
     Content_Type => 'form-data',
-    Content      => $form,
+    Content      => {
+      HIDDENNAME                        => $arg->{user},
+      CAN_MULTIPART                     => 1,
+      pause99_add_uri_upload            => File::Basename::basename($file),
+      SUBMIT_pause99_add_uri_httpupload => " Upload this file from my disk ",
+      pause99_add_uri_uri               => "",
+      pause99_add_uri_httpupload        => [ $file ],
+      ($arg->{subdir} ? (pause99_add_uri_subdirtext => $arg->{subdir}) : ()),
+    },
   );
 
   $request->authorization_basic($arg->{user}, $arg->{password});
@@ -96,6 +81,26 @@ sub upload_file {
   }
 }
 
+
+sub new {
+  my ($class, $arg) = @_;
+
+  $arg->{$_} or Carp::croak("missing $_ argument") for qw(user password);
+  bless $arg => $class;
+}
+
+
+sub log {
+  shift;
+  print "$_[0]\n"
+}
+
+sub debug {
+  my ($self) = @_;
+  return unless $self->{debug};
+  $self->log($_[0]);
+}
+
 1;
 
 __END__
@@ -108,7 +113,7 @@ CPAN::Uploader - upload things to the CPAN
 
 =head1 VERSION
 
-version 0.001
+version 0.002
 
 =head1 WARNING
 
@@ -128,19 +133,47 @@ into this module.
 
 =cut 
 
+=cut 
+
 =head1 METHODS
 
+=head2 upload_file
+
+    CPAN::Uploader->upload_file($file);
+
+    $uploader->upload_file($file, \%arg);
+
+Valid arguments are:
+
+    user     - (required) your CPAN / PAUSE id
+    password - (required) your CPAN / PAUSE password
+    subdir   - the directory (under your home directory) to upload to
+    debug    - if set to true, spew lots more debugging output
+
+This method attempts to actually upload the named file to the CPAN.  It will
+raise an exception on error.
+
+=head2 new
+
+    my $uploader = CPAN::Uploader->new(\%arg);
+
+This method returns a new uploader.  You probably don't need to worry about
+this method.
+
+Valid arguments are the same as those to C<upload_file>.
+
 =head2 log
 
     $uploader->log($message);
 
+This method logs the given message by printing it to the selected output
+handle.
+
 =head2 debug
 
     $uploader->debug($message);
 
-=head2 upload_file
-
-    $uploader->upload_file($file, \%arg);
+This method logs the given message if the uploader is in debugging mode.
 
 =head1 AUTHOR
 
